/*****************************************************************
	T/SDAS-SCOM2 demo utility for TORNADO-54x.
	demo functions: Dual-channel Analog I/O.

	(C) MicroLAB Systems, 1998


	notes:
	  1. BSP/McBSP0 SIO port is used for communication with
             T/SDAS-SCOM2 DCM.
	  2. DSP software handles C54x DSPs with both BSP and McBSP
	     SIO ports and automatically configures SIO port.
	     Detection of C54xx DSPs (C5402/C5410/C5416) with McBSP ports
	     is performed by host software and information is passed
	     via the MCD->mcbsp_flag of MCD communication descriptor.

******************************************************************/

#include	<math.h>
#include	"c54x_def.h"
#include 	"t54x_dsp.h"


struct	MCD_DD
	{ int	m_id,
		prg_id,
                status,
                err,
		cmnd;
	  int	fq;
	  int   epoch_len,
                adc_data_ptr,
                dac_data_ptr,
		chan ,

		is,             // AC01 input selector
		ra,		// AC01 RA/RB params
		rb,
		in_gain,	// AC01 input gain
		out_gain,	// AC01 output gain
		mcbsp_flag	// LSB==0 for C54x with BSP port (C542/C548/C549)
				// LSB==1 for C54xx with McBSP ports (C5402/C5410/C5416)
                ;
	} ;

struct	MCD_DD *MCD	= (struct MCD_DD *)0xff00;


#define	T54X_ID			0x101
#define	PRG_ID			0x100

#define	MSTAT_RDY		0 /* Master status values */
#define	MSTAT_BUSY		1
#define	MSTAT_ATTN		2
#define	MSTAT_GO		3
#define	MSTAT_ABORT		4

#define	MERR_OK			0 /* Master returned error codes */
#define	MERR_INV_CMND		1 /*     ; invalid command */
#define	MERR_INV_ATTN		2

#define	MCMND_NOP		0
#define	MCMND_ABORT		1

#define	MCMND_DAS_SCAN		0x100
#define	MCMND_DAS_DAC_SCAN	0x101
#define	MCMND_DAC_SING_LOAD	0x102
#define	MCMND_ADC_TO_DAC	0x103

#define	ML_OFF			0
#define	ML_ON			1

#define	MDATA_BUF_LEN		0x3c00L /* maximum data array length */


extern  unsigned int r_buf_start, x_buf_start;
int	*ADC,	*DAC;		/* ptrs to the ADC/DAC data */

int	host_rq_flag=0;

int	mcbsp_flag=0;		// is set to '1' for T54xx with McBSP ports

#define	SIO_NOT_MCBSP	(mcbsp_flag==0)
#define	SIO_MCBSP	(mcbsp_flag==1)

#include	"d5_c2_a2.i"



interrupt void c_xint3()   /* H->M IRQ */
{
	 host_rq_flag=1;
}

cpu_init()
 {
  *C54X_IMR=0;		/* clear IMR since it is not cleared on reset */
  *C54X_IFR=0;

  T54X_init_dsp();
 }





int	cmnd_decoder()			/* command decoder	*/
 {
	if(MCD->status!=MSTAT_ATTN) return(MERR_INV_ATTN);

	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
	   case MCMND_NOP:		/* common cmnds */
	   case MCMND_ABORT:
		 return(MERR_OK);

                                        /* application specific commands */
	   case MCMND_DAS_DAC_SCAN:
	   	 return( adc_dac_scan() );

	   case MCMND_ADC_TO_DAC:
	   	 return( adc_dac_scan() );

	   default:			/* all unknown commands cause error */
		 return( MERR_INV_CMND );
	 }

 }



void main()
{
	cpu_init();

	ADC	=   (int *) &r_buf_start ;
	DAC	=   (int *) &x_buf_start ;

	MCD->m_id		= T54X_ID;
	MCD->prg_id		= PRG_ID;
	MCD->cmnd		= 0;
	MCD->fq			= 0;
	MCD->epoch_len		= 0;
	MCD->err		= MERR_OK;
	MCD->adc_data_ptr	= (unsigned) ADC;
	MCD->dac_data_ptr	= (unsigned) DAC;
	MCD->status		= MSTAT_BUSY;

	mcbsp_flag = (MCD->mcbsp_flag & 1);	// set McBSP detector

	system_init();
 	MCD->status=MSTAT_RDY;

	while(1)			/* continuous loop	*/
	 {
	   host_rq_flag=0;		/* poll for the HOST RQ */

           C54X_XINT3_CLEAR;
	   C54X_XINT3_ENABLE;  /* enable IRQ-3 (HM_RQ) */

           while(host_rq_flag==0) ;
	   C54X_XINT3_DISABLE;		/* disable HOST RQ	*/

	   MCD->err = cmnd_decoder();	/* call exec & return error code */
           MCD->status = MSTAT_RDY;	/* clear status flag */
	 }
}
