/*
        Application specific execution module.
	device: TORNADO-54x with T/SDAS-SCOM2
	demo functions: Dual-channel Analog I/O
*/


typedef struct
	{ unsigned int ctrl :2; /* Control Bits     */
	  unsigned int r_w  :1; /* Read/!Write mode */
	  unsigned int addr :5; /* Register Address */
          unsigned int data :8; /* Register data    */
	} AIC_CMND_0;

AIC_CMND_0	aic_cmnd_0,
		aic_cmnd15,
		aic_cmnd1,
		aic_cmnd2,
		aic_cmnd3,
		aic_cmnd4,
		aic_cmnd5,
		aic_cmnd6,
		aic_cmnd7,
		aic_cmnd8;


		/* X/R interrupt handler parameters	*/

int	dac_scan_flag		= ML_OFF,
	dac_cyclic_scan_flag	= ML_ON,
	dac_data_start_ptr,
        dac_get_ptr,
	dac_data_count,
        dac_data_count_max,

	master_cmnd		= 0,
	slave_cmnd		= 0,
	program_aic		= ML_ON
         ;

int	adc_scan_flag		= ML_OFF,
	adc_cyclic_scan_flag	= ML_OFF,
	adc_save_ptr,
	adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;


int r_data, din;

interrupt void c_bsp_rint()   /* BSP receiver interrupt */
{
	unsigned   r_fs;

	r_data= (SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR) ;
	r_fs = r_data & 0x3 ;

		/*
	   	    master primary     - r_fs=0
		    slave  primary     - r_fs=1
		    master secondary   - r_fs=2
		    slave  secondary   - r_fs=3
		*/

	switch(r_fs)
	    {
		case 0:
		   {
		   	if(adc_scan_flag==ML_ON)
			 {
			   r_data= (r_data )>>2;
                           *((int *)(adc_save_ptr))=r_data;
			 }
		   }
		    break ;
		case 1:

                     if(adc_scan_flag==ML_ON)
			  {
			   r_data= (r_data )>>2;
                           *((int *)(adc_save_ptr+adc_data_count_max))=r_data;
                           adc_save_ptr++;
                           if(++adc_data_count >=adc_data_count_max)
				 { if(adc_cyclic_scan_flag==ML_ON)
				      {
					adc_data_count=0;
                                        adc_save_ptr=adc_data_start_ptr;
				      }
				      else
				        adc_scan_flag=ML_OFF;
				 }

			  }
		    break ;
		case 2:
		    din= ((r_data )>>12) & 0x000f;
		    break ;

		case 3:
		default:    break ;
	    }
}



/*---------------------- BSP-X/R interrupt handlers -----------------------*/

unsigned   	t_fs=0;
int		p_p=0;
unsigned        dxr;	// data to be transmitted

interrupt void c_bsp_xint()   /* BSP transmitter interrupt */
{
	dxr = 0;
	if (program_aic)
	  {
	     switch(p_p)
		{
		 case  0:
		 case  1:
		 case  2:
                          dxr=0x0;
                          break;
		 case  3:
		 case  5:
		 case  7:
		 case  9:
		 case  11:
		 case  13:
		 case  15:
		 case  17:
		 	  dxr=0x3;
                          break;
		 case  4:
		 	  dxr=*((unsigned *) &aic_cmnd8);	
                          break;
		 case  6:
		 	  dxr=*((unsigned *) &aic_cmnd6);	
                          break;
		 case  8:
		 	  dxr=*((unsigned *) &aic_cmnd1);	
                          break;
		 case 10:
		 	  dxr=*((unsigned *) &aic_cmnd2);	
                          break;
		 case 12:
		 	  dxr=*((unsigned *) &aic_cmnd3);	
                          break;
		 case 14:
		 	  dxr=*((unsigned *) &aic_cmnd4);	
                          break;
		 case 16:
		 	  dxr=*((unsigned *) &aic_cmnd5);	
                          break;
		 case 18:
		 	  dxr=*((unsigned *) &aic_cmnd7);
		 	  p_p=-1;
			  program_aic=ML_OFF;
			  t_fs=0;
			  break;

	   	 default: break;
 		};
		p_p++;
	  }
	 else
	   {    /*
	   	    master primary transmit    - t_fs=0
		    slave  primary transmit    - t_fs=1
		    master secondary transmit  - t_fs=2
		    slave  secondary transmit  - t_fs=3
		*/
	     switch(t_fs)
		{
		  case 0:    if(dac_scan_flag==ML_OFF)
	 			{  		/* if DAC-scan is disabled : */
	   			 dxr=0x3;/* send data=0 and LSB[1,0]=#3 */
         			}
	 			else
	  			  {
	    			   dxr=((*((int*)dac_get_ptr))<<2) | 0x3;
				  }
			     break;

		  case 1:    if(dac_scan_flag==ML_OFF)
	 			{  		/* if DAC-scan is disabled : */
	   			 dxr=0x3;/* send data=0 and LSB[1,0]=#3 */
         			}
	 			else
	  			  {
	    			   dxr=((*((int*)(dac_get_ptr+dac_data_count_max)))<<2) | 0x3;
				     dac_get_ptr++;
				     if(++dac_data_count >=dac_data_count_max)
				       { if(dac_cyclic_scan_flag==ML_ON)
				            {
				               dac_data_count=0;
				               dac_get_ptr=dac_data_start_ptr;
				            }
				          else
				            dac_scan_flag=ML_OFF;
				       }
				  }
			     break;

		  case 2:
			     dxr=master_cmnd;
			     break;

		  case 3:
			     dxr= slave_cmnd ;
			     break;
	   	   default: break;
		}

	     t_fs=(t_fs+1) & 0x3 ;
	   }

        if(SIO_MCBSP)	//-- load DXR/DXR1 upon the DSP type
		*C54X_MCBSP0_DXR1=dxr;
	  else  *C54X_BSP_DXR=dxr;
}

void set_config_aic()
  {
        adc_cyclic_scan_flag	= ML_OFF;
	adc_scan_flag		= ML_OFF;
        dac_cyclic_scan_flag	= ML_OFF;
	dac_scan_flag		= ML_OFF;

	slave_cmnd	= 0x0;
	master_cmnd	= 0x0;

		aic_cmnd1.ctrl=0;
		aic_cmnd1.r_w=0;
		aic_cmnd1.addr=1;
		aic_cmnd1.data	= MCD->ra & 0xff;

		aic_cmnd2.ctrl=0;
		aic_cmnd2.r_w=0;
		aic_cmnd2.addr=2;
		aic_cmnd2.data	= MCD->rb & 0xff;

		aic_cmnd3.ctrl=0;
		aic_cmnd3.r_w=0;
		aic_cmnd3.addr=3;
		aic_cmnd3.data =0 ;

		aic_cmnd4.ctrl=0;
		aic_cmnd4.r_w=0;
		aic_cmnd4.addr=4;
		aic_cmnd4.data = ((((MCD->in_gain)+1)<<2)+((MCD->out_gain)+1)) & 0xf ;

		aic_cmnd5.ctrl=0;
		aic_cmnd5.r_w=0;
		aic_cmnd5.addr=5;
		aic_cmnd5.data = ((MCD->is)+1) & 0x3 ;

		aic_cmnd6.ctrl=0;
		aic_cmnd6.r_w=0;
		aic_cmnd6.addr=6;
		aic_cmnd6.data =0x24 ;

		aic_cmnd7.ctrl=0;
		aic_cmnd7.r_w=0;
		aic_cmnd7.addr=7;
		aic_cmnd7.data =20 ;

		aic_cmnd8.ctrl=0;
		aic_cmnd8.r_w=0;
		aic_cmnd8.addr=8;
		aic_cmnd8.data =2 ;

		aic_cmnd15.ctrl=0;
		aic_cmnd15.r_w=0;
		aic_cmnd15.addr=15;
		aic_cmnd15.data = 0x50 ;

	t_fs=0;
	p_p=0;

  }


void init_sio()		/* T54x SIO-port initialization */
 {

   if(SIO_MCBSP)	//-- configure C54xx DSPs with McBSP ports
    {
 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0000;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0000;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP0_PCR   = 0x000c;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   *C54X_MCBSP0_RCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   *C54X_MCBSP0_RCR2  = 0x0004;

	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP0_XCR1  = 0x0040;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP0_XCR2  = 0x0004;

		//-- now enable McBSP port operation
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP0_SPCR1 = 0x0001;
	   *C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP0_SPCR2 = 0x0001;

	   *C54X_MCBSP0_DXR1 = 0  ;		/* perform transmission of false word to init transmitter OK */
    }
     else	//-- configure C548/C549/C5402 DSP with BSP/TDM ports
    {
	   *C54X_BSP_SPC=0x0008;	/* reset BSP */
	   *C54X_BSP_SPCE=0x0000;	/* set FSX=high, CLKX=high=external */
	   *C54X_BSP_SPC=0x00c8;	/* configure BSP */

	   *C54X_BSP_DXR = 0  ;		/* perform transmission of false word to init transmitter OK */
    }

 }


void system_init()
{
	T54X_set_pxsx_run(0x7);	// remove SIOX/PIOX reset

	set_config_aic();
	program_aic=ML_ON;

	init_sio();

	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_ENABLE;
	C54X_BSPR_INT_ENABLE;

	C54X_INT_ENABLE;
	while(program_aic);
}


/****************************************************************************/

	/* this procedure performs cyclic DAC output and waits for
	   host to enable ADC scanning. Then it saves ADC data and terminates.
	*/

int	adc_dac_scan()
  {
				/* set X/R interrupt driver parameters */
	adc_data_start_ptr	=  (unsigned)ADC;
        adc_save_ptr		=  (unsigned)ADC;

        dac_data_count_max	= MCD->epoch_len;
        adc_data_count_max	= MCD->epoch_len;
	dac_data_count		= 0;
	adc_data_count		= 0;


	if (MCD->cmnd==MCMND_ADC_TO_DAC)
	  {                       /* settings for ADC->DAC scanning */
	    dac_data_start_ptr		= (unsigned)ADC;
            dac_get_ptr			= (unsigned)ADC;

	    adc_cyclic_scan_flag	= ML_ON;
	    adc_scan_flag		= ML_ON;
	  }
	else
	  {
	    dac_data_start_ptr	= (unsigned)DAC;
            dac_get_ptr		= (unsigned)DAC;

	    adc_cyclic_scan_flag	= ML_OFF;
	    adc_scan_flag		= ML_OFF;
	  }

	dac_cyclic_scan_flag	= ML_ON;
	dac_scan_flag		= ML_ON;


			/* wait for MCD->status==GO or ABORT*/
	while((MCD->status!=MSTAT_GO) && (MCD->status!=MSTAT_ABORT));

	if ( MCD->status==MSTAT_GO )
	  {
        	adc_scan_flag	= ML_ON;	/* enable saving ADC -data */
		while(adc_scan_flag!=ML_OFF);	/* wait until ADC scanning will OFF */
	  }
	dac_scan_flag	= ML_OFF;	/* disable DAC transmission */
	adc_scan_flag	= ML_OFF;

	return(MERR_OK);
  }
