/***********************************************************************
 Dual-channel Analog I/O Demo for TORNADO-54x with T/SDAS-SCOM2.
 Copyright (C) MicroLAB(tm) Systems Ltd, 1998-2000

 demo description:
   1. This demo provides dual-channel DAC/ADC signal acquisition using
      either dual-channel analog I/O cable set or splitter with two single-
      channel cable sets.
   2. C54x/C54xx DSP type is automatically recognized in order
      to arrange BSP/McBSP port configuration of T54x.

 available demo options:
   0 : Output F0/2xF0 sine waves via DAC-1/DAC-2
   1 : Record Ch#1/Ch#2 data in on-board SRAM
   2 : Cyclic Playback Ch#1/Ch#2 data from on-board SRAM
   3 : Continuous pass-through (ADC-1=>DAC-1,ADC-2=>DAC-2)

*************************************************************************/



#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <mlu.h>
#include <ips_ut.h>
#include <t54x_h.h>
#include "t54_comm.i"

#define  SCAN_LEN_DEFAULT     (0x3c00L/2L)	// assume that 2 DAC and 2 ADC buffers are used

#define  SCAN_LEN_MIN   1L
#define  SCAN_LEN_MAX   200000L


long   scan_len=0L,   // epoch length in samples
       kbd_cmnd,cmnd;       // amplitude

long	i,chan=3;
int	tmp,ex_status, fq_code=1, in_gain=0, out_gain=1; //,adc_scan_flag=0
char	s[80],ch;
int	dat_type=2; // def for INT type (number of bytes per word)

MLU_TBDD TB;  // time bell descriptor


#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");


struct MCD_DD  *MCD;     // MCD pointer

int   *D;               // master data buffer array
long       MADC_SADDR, MDAC_SADDR;

unsigned is=0,ra,rb;

struct	MCD_STLC_DD
	{
	  unsigned	is;		// input selector (0-main, 1-aux)
	  unsigned  	ra,rb;  	// internal Codec clock counters that define sampling freq
	  unsigned	in_gain;
  	  unsigned	out_gain;
	  unsigned 	mcbsp_flag;	// ==1 for T54x with McBSP ports (T54xx: T5402/T5410/T5416)
					// ==0 for T54x with BSP ports (T542L/T548/T549)
	}
		*MCD_STLC ;



void	cycle_pause()
 { long i;

 	for(i=0;i<1000000;i++);
 }


inp_param()
{

  enter_sf:
      printf("Speech bandwidth (Fb) and sampling (Fs) frequencies selection: \n");
      printf("     0: 7.2k/16k \n");
      printf("     1: 3.6k/8k\n");

      printf("enter Fs code #           : ");
      fq_code=(int)getlong(0L,1L,(long)fq_code,CR_ENABLE,ESC_MASK,&ex_status);

      if(ex_status==ESC_MASK)
       {aborted:
	  printf(" <aborted>\n");
	  return(1);
       }

      switch(fq_code)
       {
         case 0: ra=16L;rb=18L;	break;
         case 1: ra=32L;rb=18L;	break;
	 default: break;
       }

      printf("input signal scale: \n");
      printf("     0:  0dB  (+-1.5v) \n");
      printf("     1: +6dB  (+-0.75v)\n");
      printf("     2: +12db (+-0.375v)\n");

      printf("enter input scale code    : ");
      in_gain=(int)getlong(0L,2L,(long)in_gain,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("output signal scale: \n");
      printf("     0:  0dB  (+-3v) \n");
      printf("     1: -6dB  (+-1.5v)\n");
      printf("     2: -12db (+-0.75v)\n");

      printf("enter output scale code   : ");
      out_gain=(int)getlong(0L,2L,(long)out_gain,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("input select(0-main/1-aux): ");
      is=getlong(0L,1L,is,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      printf("\n");
      return(0);
}


/*****************************************************************************
// fill on-board SRAM with zero or sine waves with F0 and 2xF0 frequencies
*****************************************************************************/

#define FILL_SINE   1
#define FILL_ZERO   0

 int __fill_dac_sram(int fill_mode)
  {
    long count, start, q, n, n1, j, chan_count, w_len;
    unsigned pmap_old;

      pmap_old=T54X_SMP_RG_DATA;

      CLEAR_CURRENT_LINE;
      printf("filling SRAM ...\r");

      for (chan_count=0; chan_count<2; chan_count++)
       {
	    start=MDAC_SADDR/dat_type;
  	    j=start;
  	    if(chan_count==1) j+=scan_len;
	    count=scan_len;
	    n=0L;

	    do
	      {
                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++)
		  {
		    if(fill_mode==FILL_SINE)
		      {
		        if(chan_count==0)
			    *D++ = (int) ((double) (5000.0 * sin(500.0*M_PI*((double)(n1+n))/((double) scan_len))));
		          else
			    *D++ = (int) ((double) (5000.0 * sin(1000.0*M_PI*((double)(n1+n))/((double) scan_len))));
		      }
		     else
			  *D++ = 0L;
		  }

                if(T54X_SB_ERR_DATA)
	           {
		     CLEAR_CURRENT_LINE;
                     printf("error: SB error.Fill mode: %d",fill_mode);
		     printf("\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
		n+=q;
	      }
	     while (count>0);
         }
     T54X_SET_SMP_RG(pmap_old);
     CLEAR_CURRENT_LINE;
     return(0);
  }



 int __read_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    int *LPTR;
    long j, count,n1,q, w_len;


            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *LPTR++= *D++);

                if(T54X_SB_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error r\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


 int __write_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    int *LPTR;
    long j, count,n1,q, w_len;


            LPTR=(int *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {

                T54X_set_smp_ptr(j*dat_type,T54X_DRAM_AREA,&D);
		w_len=(T54X_SMP_LEN/dat_type)-j%(T54X_SMP_LEN/dat_type);
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *D++=*LPTR++ );

                if(T54X_SB_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: SB error w\n\n");
           	     return(1);
	           }
		count-=q;
		j+=q;
	      }
	     while (count>0);
       return(0);

  }


#define TMP_BUF_LEN	0x8000L


 int __move_adc_to_dac(long buf_len)
  {
    long count, j, q, jd;
    int *TMP;



      CLEAR_CURRENT_LINE;
      printf("moving SRAM data ...\r");


    if((TMP=(int *) halloc(TMP_BUF_LEN*dat_type,1))==NULL)

       { printf("error: out of memory\n\n");
	 return(1);
       }

                      /* copy ADC data to DAC buffer */

	    j=MADC_SADDR/dat_type;
	    jd=MDAC_SADDR/dat_type;
	    count=buf_len;

	    do
	      {
                if(count>=TMP_BUF_LEN) q=TMP_BUF_LEN;
		    else	q=count;
		if(   __read_sb_data(j,q,TMP)!=0
		   || __write_sb_data(jd,q,TMP)!=0
		  )
           	   {
                     hfree(TMP);return(1);
	           }
		count-=q;
		j+=q;
		jd+=q;
	      }
	     while (count>0);


    CLEAR_CURRENT_LINE;
    hfree(TMP);
    return(0);
  }



int do_cmd(int adc_scan_enable)
 {
  T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);

  MCD->cmnd = cmnd;
  if (cmnd==MCMND_ADC_TO_DAC)	
        MCD->epoch_len = 0x10;
   else	MCD->epoch_len = scan_len;

  MCD->status = MSTAT_ATTN;

  T54X_SET_HM_RQ;  // set MASTER ATTN via interrupt

  printf(" T54x operation in progress. Press <SPACE> to terminate or <ESC> to abort...\r");

  do                      // wait for the ESC pressed or data ready
   {

     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
 	      exe_aborted:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, T54X reset>\n");
	      m_restart:
	         T54X_SET_MRESET;
		 T54X_CLR_SB_ERR;
	         return(1);

            case 0x20:  // issue ADC SCAN synch
                 if (adc_scan_enable)
		  {
	    	      { CLEAR_CURRENT_LINE;
		        printf("\rADC scanning...\r");
		        MCD->status=MSTAT_GO;
		      }
		  }
		  else
		  {
		     CLEAR_CURRENT_LINE;
		     MCD->status=MSTAT_ABORT;
		     printf("\r Operation is aborted...\r");
		  }
		 break;
            default:
		 break;
	  }
       }
   }
    while(MCD->status!=MSTAT_RDY);// && (MCD->err==MLU_OK));
//    cycle_pause();
    CLEAR_CURRENT_LINE;

   if (MCD->err!=MLU_OK)
     { CLEAR_CURRENT_LINE;
       printf("error: %xd\n", MCD->err);
       return (1);
     }
   return(0);
 }



main()

{ int r;
  char err_mess[100], dev_name[100];
  unsigned dev_id;


   printf("\nAnalog I/O Demo for TORNADO-54x with T/SDAS-SCOM2 DCM. v.1.2\n");
   printf("(C) MicroLAB Systems Ltd, 1998-2000\n\n");

   if(   (r=T54X_open(0x350))!=T54X_OK
      || (r=T54X_dev_id(&dev_id))!=T54X_OK
      || (r=T54X_dev_name(dev_name))!=T54X_OK
     )
     {  error:
          T54X_last_err_message(err_mess);
	  printf("error: %s\n", err_mess);
	  goto exit;
     }

   printf("DSP system:  %s, s/n: #%x\n\n", dev_name, dev_id & 0xff);

   T54X_set_smp_ptr(MCD_SADDR,T54X_DRAM_AREA,&MCD);
   MCD_STLC = (struct MCD_STLC_DD*) (& ((char*)&(MCD->chan))[dat_type]);

   T54X_SET_MRESET;       // init T54x and validate the response code
   T54X_CLR_SB_ERR;

   if(inp_param()!=0) goto exit;

      MCD->m_id=0xffff;
      MCD->prg_id=0xffff;
      MCD->status=0xffff;
      MCD->err=0xffff;
      MCD->adc_data_saddr=0xffff;
      MCD->dac_data_saddr=0xffff;

	MCD_STLC->is		= is;
	MCD_STLC->ra		= ra;
  	MCD_STLC->rb		= rb;
  	MCD_STLC->in_gain	= in_gain;
  	MCD_STLC->out_gain	= out_gain;


		//--- pass info about the C54x/C54xx DSP type to the DSP
		//    resident software in order to provide correct arrangement
		//    of BSP/McBSP SIO port management

	MCD_STLC->mcbsp_flag	= (   T542L_BOARD
				    | T548_BOARD
				    | T549_BOARD) ? 0:1;

   MLU_set_bell(1,&TB);  // set time control =1s
   T54X_SET_MGO;

   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
       if(MLU_check_bell(&TB)<0)
	   { printf("error: no reply from TORNADO-54x\n\n");
	     goto exit;
	   }
       if(T54X_SB_ERR_DATA!=0)
	   { printf("error: SB error  main\n\n");
	     goto exit;
	   }
     }

   if(   MCD->m_id!=T54X_M_ID    // validate MCD returned
      || MCD->prg_id!=T54X_PRG_ID
      || MCD->status!=MSTAT_RDY
      || MCD->cmnd!=0
      || MCD->adc_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->adc_data_saddr>=(T54X_DRAM_EADDR-2*SCAN_LEN_DEFAULT)
      || MCD->dac_data_saddr<=T548_DSP_ONCHIP_DARAM_EADDR
      || MCD->dac_data_saddr>=(T54X_DRAM_EADDR-2*SCAN_LEN_DEFAULT)
     )
       { printf("error: invalid T54x initialization data\n");
	 goto exit;
       }

   MADC_SADDR = ((long)MCD->adc_data_saddr)*dat_type;
   MDAC_SADDR = ((long)MCD->dac_data_saddr)*dat_type;
   scan_len=SCAN_LEN_DEFAULT;
   __fill_dac_sram(FILL_ZERO);


while(1)
 {
   printf("\n commands : \n");
   printf("      0 : Output F0/2xF0 sine waves via DAC-1/DAC-2\n");
   printf("      1 : Record Ch#1/Ch#2 data in on-board SRAM\n");
   printf("      2 : Cyclic Playback Ch#1/Ch#2 data from on-board SRAM\n");
   printf("      3 : Continuous pass-through (ADC-1=>DAC-1,ADC-2=>DAC-2)\n");
   printf("    Esc : exit\n\n");

   printf("enter command: ");


kbd_loop:
   switch(read_char())
     {
        case '0':
                CLEAR_CURRENT_LINE;
		printf("Output F0/2xF0 sine waves via DAC-Ch#1/Ch#2 >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
		scan_len=SCAN_LEN_DEFAULT;

		if (    __fill_dac_sram(FILL_SINE)!=0
                     ||  do_cmd(MLU_OFF)!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
                   ) goto exit;
		break;


	case '1':

                CLEAR_CURRENT_LINE;
		printf("Record Ch#1/Ch#2 data in on-board SRAM >>>>\n");
		cmnd = MCMND_DAS_DAC_SCAN;

		scan_len=SCAN_LEN_DEFAULT;

		if (    __fill_dac_sram(FILL_ZERO)!=0
                     || do_cmd(MLU_ON)!=0
		     || __move_adc_to_dac(2L*SCAN_LEN_DEFAULT)!=0
                   ) goto exit;
		break;




	case '2':
                CLEAR_CURRENT_LINE;
		printf("Play Ch#1/Ch#2 data from on-board SRAM >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
		scan_len=SCAN_LEN_DEFAULT;

		if ( do_cmd(MLU_OFF)!=0
                   ) goto exit;
		break;



	case '3':

                CLEAR_CURRENT_LINE;
		printf("Continuous Pass-through Ch#1/Ch#2 data (ADC->DAC) >>>>\n");
		cmnd = MCMND_ADC_TO_DAC;

		if (do_cmd(MLU_OFF)!=0) goto exit;
                break;


        case KEY_ESC:
	  printf(" <aborted>\n");
	  goto exit;

	default:
		beep_err();
		goto kbd_loop;
     }

 }
exit:
  T54X_SET_MRESET;
  printf("\n");

}