/***************************************************************
	Application specific execution module.
	DCM: 			T/SDAS-SCOM2
	DSP platform: 	C54xx
	modes: 			D/A SINE OUTPUT, A/D->D/A pass-through,
					all modes utilize 2-ch D/A and 2ch A/D
****************************************************************/

/*--------- MACROS for real-time performance tracking ---------
	These MACROS are used for real-time performance tracking
	(consumed tme) for execution of RECEIVER/TRANSMITTER
	interrupts.

        The tracking signal is XIOF-0/TM signal from the SIOX site connector.
	This shared signal should be configured by on-board jumper J3
	as the XIOF-0 signal (T54x rev.2A and earlier).

	In the beginning of RECEIVER and TRANSMITTER interrupts the
	tracking signal is set to '1' and is cleared on exit from
	interrupt.

	In order to see the consumed time for execution of
	RECEIVER/TRANSMITTER interrupts, connect the
	oscilloscope probe to pin XIOF-0 of SIOX connector.
	(refer to TORNADO-54x documentation re SIOX signal specs).

*/

#define _TIME_TRACKING0_SET_0	_SET_XIOF0_OFF_
#define _TIME_TRACKING0_SET_1	_SET_XIOF0_ON_

#define _TIME_TRACKING1_SET_0	_SET_XIOF1_OFF_
#define _TIME_TRACKING1_SET_1	_SET_XIOF1_ON_



// McBSP port access macros
// Modify these macros in case different serial port is used (as BSP1 or McBSP-1/2, etc)
#define	_SIO_XDATA(x)		if(SIO_MCBSP) (*C54X_MCBSP0_DXR1)=x;else (*C54X_BSP_DXR)=x;
#define	_SIO_RDATA			((SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR))


// TLC320AC01 AIC command data word
//   Declaration of TLC AIC command data structure.
//
//   Note:
//	  TI C54x C Compiler processes bit fields in the reverse manner re C3x/C6xxx compiler,
//	  i.e. first declared fields appear in the most significant bits of the structure
//	  word. (see TI C54x C Compiler UG, section 5.1.7 (Field/Structure Alignment).
//	  Note that TI C3x C Compiler provides bit fields processing in the
//	  direct manner (!!!).
typedef struct
{
	unsigned ctrl :2; 	// Control Bits
  	unsigned r_w  :1; 	// Read/Write mode
	unsigned addr :5; 	// Register Address
    unsigned data :8; 	// Register data
} AIC_CMND;


// AIC secondary dataword commands.
// Suffix for each command corresponds to the AIC register #.
AIC_CMND
		aic_cmnd0,
		aic_cmnd1,
		aic_cmnd2,
		aic_cmnd3,
		aic_cmnd4,
		aic_cmnd5,
		aic_cmnd6,
		aic_cmnd7,
		aic_cmnd8,
		aic_cmnd15;


// X/R interrupt handler parameters
int		dac_scan_flag			= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_count,
    	dac_data_count_max,
		program_aic				= ML_ON ;

unsigned
		master_cmnd				= 0,
		slave_cmnd				= 0;


int		adc_scan_flag			= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_data_count,
		adc_data_count_max ;

int		*adc_ptr			=NULL,	// run-time pointers to ADC/DAC data
        *dac_ptr			=NULL;



//------------ receiver interrupt handler --------------------------------
int r_data, din;

interrupt void sio_rint()
{
	unsigned   r_fs;

   _TIME_TRACKING0_SET_1;		// set PERFORMACE TRACKING FLAG

	// read BSP/McBSP data register
	r_data= ((SIO_MCBSP) ? (*C54X_MCBSP0_DRR1):(*C54X_BSP_DRR));

	// unmask  control bits for data interpreting
	r_fs = r_data & 0x3 ;

	// available data type:
	// 	    master primary     - r_fs=0
	//	    slave  primary     - r_fs=1
	//	    master secondary   - r_fs=2
	//	    slave  secondary   - r_fs=3


	switch(r_fs)
    {
		case 0:
		   	if(adc_scan_flag==ML_ON)
			{
				// A/D channel-0 data
			   r_data >>= 2;
               adc_ptr[adc_data_count]=r_data;
			}
		    break ;

		case 1:
			if(adc_scan_flag==ML_ON)
		  	{
				// A/D channel-1 data
				r_data >>= 2;
                adc_ptr[adc_data_count+adc_data_count_max]=r_data;

                if(++adc_data_count >=adc_data_count_max)
				{
					if(adc_cyclic_scan_flag==ML_ON)
						adc_data_count=0;
				    else
				        adc_scan_flag=ML_OFF;
				}
			}
		    break ;

		case 2:
			// DIN data data
		    din= ((r_data )>>12) & 0x000f;
		    break ;

		case 3:
		default:
			break ;
    }

   _TIME_TRACKING0_SET_0;        	// clear PERFORMACE TRACKING FLAG

}
//--------------------------------------------------------------------



//-------------- transmitter interrupt handler -----------------------

unsigned   	t_fs=0;
int			p_p=0;
unsigned    dxr;		// data to be transmitted

interrupt void sio_xint()
{
   _TIME_TRACKING1_SET_1;		// set PERFORMACE TRACKING FLAG

	dxr = 0;

	if (program_aic)
	{
		// AIC program session. Scan over all AIC command registers.
	  	switch(p_p)
		{
			// these time slots are used to get into AIC synchronization
		 	case  0:
		 	case  1:
		 	case  2:
				dxr=0x0;
                break;

			// odd time slots are data time slots,
			// set 0x3 in order to requst secondary communcation
		 	case  3:
		 	case  5:
		 	case  7:
		 	case  9:
		 	case  11:
		 	case  13:
		 	case  15:
		 	case  17:
		 	  	dxr=0x3;
                break;

			// even time slots are secondary communication for AIC register writes
		 	case  4:
		 	  	dxr=*((unsigned *) &aic_cmnd8);
                break;

		 	case  6:
		 	  	dxr=*((unsigned *) &aic_cmnd6);
                break;

		 	case  8:
		 	  	dxr=*((unsigned *) &aic_cmnd1);
                break;

		 	case 10:
		 	  	dxr=*((unsigned *) &aic_cmnd2);
                break;

		 	case 12:
		 	  	dxr=*((unsigned *) &aic_cmnd3);
                break;

		 	case 14:
				dxr=*((unsigned *) &aic_cmnd4);
                break;

		 	case 16:
		 	  	dxr=*((unsigned *) &aic_cmnd5);
                break;

		 	case 18:
				// program FSYNC split delay for SLAVE AIC
				// FSYNC is splitted to MASTER and SLAVE primary/secondary after
				// reception of this command.
		 	  	dxr=*((unsigned *) &aic_cmnd7);

				// notify that AIC programming is finished
				// set parameters
		 	  	p_p=-1;
			  	program_aic=ML_OFF;
			  	t_fs=0;
			  	break;

	   	 	default:
				break;
 		};

		p_p++;
	}
	else
	{
		//	Data transfer session.
		//	Transmit data words (primary and secondary) and secondary false
		//	command data word (AIC regsiter #0, which is MASTER/SLAVE command).
		//
		//	Data is interpreted D0/D1 using control bits:
		//		master primary transmit    - t_fs=0
		//  	slave  primary transmit    - t_fs=1
		//  	master secondary transmit  - t_fs=2
		//  	slave  secondary transmit  - t_fs=3

		switch(t_fs)
		{
			case 0:
				if(dac_scan_flag==ML_OFF)
	 			{
					// if DAC-scan is disabled
	   			 	dxr=0x3;	// send data=0 and D1/D0=[1,1]
         		}
	 			else
				{
					// primary data (DAC-0 channel)
	    			dxr= ((dac_ptr[dac_data_count])<<2) | 0x3;
				}
				break;


		  	case 1:
				if(dac_scan_flag==ML_OFF)
	 			{
					// if DAC-scan is disabled
	   			 	dxr=0x3;	// send data=0  and D1/D0=[1,1]
         		}
	 			else
	  			{
					// secondary data (DAC-1 channel)
	    			dxr= ((dac_ptr[dac_data_count+dac_data_count_max])<<2) | 0x3;

					if(++dac_data_count >=dac_data_count_max)
				    {
						if(dac_cyclic_scan_flag==ML_ON)
				        	dac_data_count=0;
				        else
				            dac_scan_flag=ML_OFF;
				   	}
				}
				break;


		  	case 2:
				dxr=master_cmnd;
			    break;


		  	case 3:
			 	dxr= slave_cmnd ;
			    break;


			default:
				break;
		}

		// increment control bits
		t_fs = ((t_fs+1) & 0x3);

	}

	// transmit data
	_SIO_XDATA(dxr);

	_TIME_TRACKING1_SET_0;        	// clear PERFORMACE TRACKING FLAG
}
//-------------------------------------------------------------------------


//----------------- create AIC configuration parameters ---------------------------
void create_aic_configuration()
{
	// reset default master/slave commands for acquisition modes
	// slave command is just NOP
	slave_cmnd	= 0;

	// MASTER command is to set all DOUT-0..3 to the '1' state. This will allow
	// to connect external XTLI option to any channel of SCOM2 DCM and set it to the
	// OFF-HOOK mode thus allowing to directly connect to the PSTN line and make it
	// operational. The user may want to set MASTER_CMND var dynamically in the infiniet
	// loop in background during SINE OUTPUT or ADC-to-DAC demo in order to perform
	// any specific operation with external XTLI/XTSI options for each SCOM2 channel.
	master_cmnd	= 0x0ff0;

	// fill-in AIC command registers
	// Note:
	//		some of AIC command registers are filled in by defualt parameters,
	// 		which can be changed upon customer appilcations. Refer to TLC320AC01
	//		AIC manual for more details.

	aic_cmnd1.ctrl	=0;
	aic_cmnd1.r_w	=0;
	aic_cmnd1.addr	=1;
	aic_cmnd1.data	= MCD->aic_ra & 0xff;

	aic_cmnd2.ctrl	=0;
	aic_cmnd2.r_w	=0;
	aic_cmnd2.addr	=2;
	aic_cmnd2.data	= MCD->aic_rb & 0xff;

	aic_cmnd3.ctrl	=0;
	aic_cmnd3.r_w	=0;
	aic_cmnd3.addr	=3;
	aic_cmnd3.data 	=0 ;

	aic_cmnd4.ctrl	=0;
	aic_cmnd4.r_w	=0;
	aic_cmnd4.addr	=4;
	aic_cmnd4.data 	= ((((MCD->aic_ain_gain)+1)<<2)+((MCD->aic_aout_att)+1)) & 0xf ;

	aic_cmnd5.ctrl	=0;
	aic_cmnd5.r_w	=0;
	aic_cmnd5.addr	=5;
	aic_cmnd5.data 	= ((MCD->aic_ain_mux)+1) & 0x3 ;

	aic_cmnd6.ctrl	=0;
	aic_cmnd6.r_w	=0;
	aic_cmnd6.addr	=6;
	aic_cmnd6.data 	=0x24 ;

	aic_cmnd7.ctrl	=0;
	aic_cmnd7.r_w	=0;
	aic_cmnd7.addr	=7;
	aic_cmnd7.data 	=20 ;	// slave frame synch delay (must be the last programmed AIC register)

	aic_cmnd8.ctrl	=0;
	aic_cmnd8.r_w	=0;
	aic_cmnd8.addr	=8;
	aic_cmnd8.data 	=2 ;	// number of FSYNC signals

	// T/SDAS-SCOM2 specific pseudo-register #15: DOUT-0..3 control (bits D4..D7)
	aic_cmnd15.ctrl	=0;
	aic_cmnd15.r_w	=0;
	aic_cmnd15.addr	=15;
	aic_cmnd15.data = 0x50 ;	// set 0x5 pattern at DOUT-0..3 outputs

}
//--------------------------------------------------------------------------



//-------------- program delay generator -----------------------------------
void __delay(n)
{ int i;
	for(i=n;i>=0;i--);
}

#define	_wait_for_serial_port_setup()	__delay(100)
#define	_wait_for_dcm_reset()			__delay(10000)
//--------------------------------------------------------------------------

//------------------- SIO port configuration -------------------------------

/*----- SIO config for DATA MODE ---------
  BSP port (C542/C548/C549):
	FSX=out XCLK_src=RCLK_src=ext
	XFSM=RFSM=contin
	XLEN=RLEN=16


  McBSP port (C5402/C5410/C5416):
	- external active high CLKR/CLKX
	- external active low FSX/FSR, no FSX/FSR delay
	- 16-bit data, active high
*/

	// BSP port config
#define	BSP_SPC_GO_MASK				0x00c0
#define	BSP_SPC_DATA				0x0008
#define	BSP_SPCE_DATA				0x0000


	// McBSP port config
#define	MCBSP_SPCR1_GO_MASK			0x0001
#define	MCBSP_SPCR2_GO_MASK			0x0001

#define	MCBSP_SPCR1_DATA			0x0000
#define	MCBSP_SPCR2_DATA			0x0000
#define	MCBSP_PCR_DATA				0x000c

#define	MCBSP_XCR1_DATA				0x0040	// XCR1 for 2x AD/DA channels
#define	MCBSP_XCR2_DATA				0x0004
#define	MCBSP_RCR1_DATA				0x0040	// RCR1 for 2x AD/DA channels
#define	MCBSP_RCR2_DATA				0x0004

#define	MCBSP_SRGR1_DATA			0x0000
#define	MCBSP_SRGR2_DATA			0x0000



void init_sio()		// DSP on-chip serial port initialization */
 {

	// configure DSP on-chip serial port upon the type of the port (BSP/McBSP)
	if(SIO_MCBSP)
    {
 		//-- configure C54xx McBSP-0 SIO port as:
		//	- external active high CLKR/CLKX
		//	- external active low FSX/FSR, no advance feature
		//	- 16-bit data, active high

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   	*C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DATA;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   	*C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DATA;

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_PCR;
	   	*C54X_MCBSP0_PCR   = MCBSP_PCR_DATA;

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR1;
	   	*C54X_MCBSP0_RCR1  = MCBSP_RCR1_DATA;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_RCR2;
	   	*C54X_MCBSP0_RCR2  = MCBSP_RCR2_DATA;

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR1;
	   	*C54X_MCBSP0_XCR1  = MCBSP_XCR1_DATA;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_XCR2;
	   	*C54X_MCBSP0_XCR2  = MCBSP_XCR2_DATA;

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   	*C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_DATA;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   	*C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_DATA;

	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   	*C54X_MCBSP0_SRGR1 = MCBSP_SRGR1_DATA;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   	*C54X_MCBSP0_SRGR2 = MCBSP_SRGR2_DATA;

		// wait some time for McBSP will enter into synchronization
		_wait_for_serial_port_setup();

   	   	// enable R/X operation
		*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   	*C54X_MCBSP0_SPCR1 = MCBSP_SPCR1_GO_MASK;
	   	*C54X_MCBSP0_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   	*C54X_MCBSP0_SPCR2 = MCBSP_SPCR2_GO_MASK;

		// wait some time for McBSP will enter into synchronization
		_wait_for_serial_port_setup();
    }
    else	//-- configure C548/C549/C5402 DSP with BSP/TDM ports
    {
	   	*C54X_BSP_SPC=BSP_SPC_DATA;		// reset BSP
	   	*C54X_BSP_SPCE=BSP_SPCE_DATA;	//set FSX=high, CLKX=high=external

		// wait some time for BSP will enter into synchronization
		_wait_for_serial_port_setup();

		// enable R/X operation
	   	*C54X_BSP_SPC=(BSP_SPC_DATA | BSP_SPC_GO_MASK);		// configure BSP

		// wait some time for McBSP will enter into synchronization
		_wait_for_serial_port_setup();
	}
 }
//----------------------------------------------------------------



//---------------- Data Acquistion procedured --------------------
// 	This procedure performs cyclic DAC output from DAC channel buffers
//	and ADC cyclic input to ADC buffers.

int	daq_run()
{
	// disable/clear DSP interrupts
	C54X_INT_DISABLE;

	// clear data acquisition control flags
	adc_cyclic_scan_flag	= ML_OFF;
	adc_scan_flag			= ML_OFF;
    dac_cyclic_scan_flag	= ML_OFF;
	dac_scan_flag			= ML_OFF;

	// reset AIC program session run-time parameters
	t_fs=0;
	p_p=0;

	// enable XMIT/RCV irqs
	// clear XMIT/RCV irq
	C54X_BSPR_INT_CLEAR;
	C54X_BSPX_INT_CLEAR;

	// generate AIC configuration parameters
	create_aic_configuration();

	// enable AIC programming
	program_aic=ML_ON;

	// remove SIOX reset signal in order to correctly synchronize to
	// AIC timing and to enable SCLK output (AIC does not provide SCLK output
	// while it is in the reset state and, therefore, McBSP port cannot be configured
	// and entered into synchronization)
	_REMOVE_DCM_RESET;

	// initialize DSP SIO port
	init_sio();

//??	// push XMIT
	//	(required to init transmitter OK due to external FSX)
	_SIO_XDATA(0);

	// wait some time for BSP/McBSP will enter into synchronization
//??	_wait_for_serial_port_setup();

	// enable XMIT/RCV irqs
	C54X_BSPX_INT_ENABLE;
	C54X_BSPR_INT_ENABLE;

	// enable global IRQs and start AIC programming procedure
	C54X_INT_ENABLE;

	// wait until AIC will be successfully programmed
	while(program_aic);


	// now prepare for data acquisition process
	// set X/R interrupt driver parameters
	// configure DAQ parameters upon compiled FUNC command
	if (MCD->cmnd==MCMND_ADC_TO_DAC)
	{
		// ADC->DAC copy with short length
	    adc_ptr					= ADC;
        dac_ptr					= ADC;

		dac_data_count			= 0;
		adc_data_count			= 0;

	    dac_data_count_max		= 0x10;
    	adc_data_count_max		= 0x10;

		dac_cyclic_scan_flag	= ML_ON;
	    adc_cyclic_scan_flag	= ML_ON;

	    adc_scan_flag			= ML_ON;
		dac_scan_flag			= ML_ON;
	}
	else
	{
		// DAC SINE output with long length
	    adc_ptr					= ADC;
        dac_ptr					= DAC;

		dac_data_count			= 0;
		adc_data_count			= 0;

	    dac_data_count_max		= MCD->epoch_len;
    	adc_data_count_max		= MCD->epoch_len;

		dac_cyclic_scan_flag	= ML_ON;
	    adc_cyclic_scan_flag	= ML_OFF;

	    adc_scan_flag			= ML_OFF;
		dac_scan_flag			= ML_ON;
	}

	// loop forever during DAQ
	while(1);
}

