/********************************************************************************
	T/SDAS-SCOM2 DCM Stand-alone Demo/Test Utility.
	for TORNADO-6x/P6x/E6x DSP Systems/Controllers. rev.1A

	(C) MicroLAB Systems Ltd, 1994-2002

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com


	Description:
	------------
	  1. This demo runs on TORNADO-6x/P6x DSP Systems and TORNADO-E6x DSP Controllers 
		 within DSP environment only in stand-alone mode and does not require 
		 host control.
	  2. This DSP demo has been designed to be user configured by means of
		 compiler run-time definitions inside SCOM2_C6X.CFG file, that allows to 
		 select different demo modes and parameters:
			- TORNADO platform selection (either TORNADO-6x, or TORNADO-P6x, 
			  or TORNADO-E6x)
			- run-time configuration parameters (sampling frequency and signal bandwidth, 
			  analog input mux, analog input gain and analog output gain)
	  3. MIRAGE-510DX/P510D JTAG emulator and TI C6000 CCS tools are required to 
		 recompile this demo and to upload code to the DSP environment.
	  4. It is possible to debug DSP code using JTAG emulator.
	  5. DSP on-chip McBSP0 SIO port is used for communication with 
		 T/SDAS-SCOM2 DCM, i.e. DCM is assumed to be instaled to on-board
		 SIOX-A rev.B site.
	  6. The demo performance is as it is due to non-optimized demo code.


	T/SDAS-SCOM2 daughter-card module installation:
	--------------------------------------------------------
	  1. T/SDAS-SCOM2 DCM must be installed into SIOX-A site at TORNADO mainboard.


	Loading Demo and Initialization:
	---------------------------------
		All demos can be loaded into DSP environment via MIRAGE JTAG emulator 
		(or compatible). 


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


*********************************************************************************/

// general includes
#include	"math.h"
#include	"stdlib.h"

// application specific includes
#include	"c6x_def.h"


// DSP master control descriptor (MCD)
// MCD is actually not required for this demo and can be just replaced by a set of
// demo parameters. However it has been enclosed for providing compatibility with
// host PC based demos for TORNADO+DCM, which used MCD to pass demo parameters from
// host PC to the TORNADO on-board DSP and to monitor demo run-time status.

struct	MCD_DD
 { 
   int 			status;		     // Master STATUS: MSTAT_RDY/BUSY/ATTN
   int 			err;        	 // command exec error

   int 			cmnd;   	     // command to exec
   unsigned 	epoch_len;       // epoch length

   unsigned 	aic_ra,aic_rb;   // AIC RA/RB on-chip dividers

   unsigned 	aic_ain_mux;     // AIC analog input MUX cgf (0 - main AIN, 1- AUX AIN)

   unsigned 	aic_ain_gain;	 // AIC analog input gain/scale
   unsigned 	aic_aout_att;	 // AIC analog output attenuation/scale
};

//	default address for MCD in SRAM
struct	MCD_DD *MCD	= (struct MCD_DD *)0x200;

// Master status values
#define	MSTAT_RDY				0	
#define	MSTAT_BUSY				1
#define	MSTAT_ATTN				2
#define	MSTAT_GO				3
#define	MSTAT_ABORT				4

// Master returned error codes
#define	MERR_OK					0 	
#define	MERR_INV_CMND			1 	//     ; invalid command 
#define	MERR_INV_ATTN			2
#define	MERR_HW_ERR				3
#define	MERR_SMALL_MEMORY_ERR	4
#define	MERR_INV_CNF_ERR		5

// Master commands defs
#define	MCMND_NOP				0
#define	MCMND_ABORT				1

#define	MCMND_DAC_SINE_OUTPUT	0x102
#define	MCMND_ADC_TO_DAC		0x103


// general defs
#define	ML_OFF					0
#define	ML_ON					1


// maximum data array length for one ADC/DAC channel (in samples)
// ADC/DAC data bufers are allocated fm heap, which is in external memory
#define	DATA_BUF_LEN			0x0400L


// global vars
int	 *ADC=NULL,*DAC=NULL;			// ptrs to the ADC/DAC data


//---------------------- include DEMO CNF file ---------------------------
#include	"scom2_c6x.cfg"	// include run-time compiler defs for demo mode and params

	// TORNADO-E6x specific definitions and include
#if(_E6X_SELECTED)
#include	"te6x_dsp.h"	// include defs for E6x

#define	_REMOVE_DCM_RESET	TE6X_set_pxsx_run(0x7)	// release SIOX-A reset
#define	_SET_DCM_RESET		TE6X_set_pxsx_run(0)	// set SIOX-A reset
#endif

	// TORNADO-6x specific definitions and include
#if(_T6X_SELECTED)
#include	"t6x_dsp.h"		// include defs for T6x

#define	_REMOVE_DCM_RESET	T6X_set_pxsx_run(0x7)	// release SIOX-A reset
#define	_SET_DCM_RESET		T6X_set_pxsx_run(0)		// set SIOX-A reset
#endif

	// TORNADO-P6x specific definitions and include
#if(_TP6X_SELECTED)
#include	"tp6x_dsp.h"	// include defs for TP6x

#define	_REMOVE_DCM_RESET	TP6X_set_pxsx_run(0x7)	// release SIOX-A reset
#define	_SET_DCM_RESET		TP6X_set_pxsx_run(0)	// set SIOX-A reset
#endif

//-----------------------------------------------------------------------

// include T/SDAS-ADDA4/16Q/200K application specific DSP code
#include	"scom2_c6x.i"	





#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
// include DAC SINE fill-in function in case DEMO FUNCTION is DAC_SINE_OUTPUT

#define	M_PI	3.141592

void fill_dac_sine()
// fill-in DAC buffers with sine data with different frequency for each channel
{
	int *D;
    unsigned  chan_count,n;

    for (chan_count=0,D=DAC; chan_count<2; chan_count++)
       {
            for(n=0;n<DATA_BUF_LEN;n++)
			{
			  switch(chan_count)
			  {
			   case 0:
			     *D++ = (int) ((float) (8000.0*sin(50.0*M_PI*((float)n)/((float) DATA_BUF_LEN))));
			      break;

			   case 1:
			     *D++ = (int) ((float) (8000.0*sin(100.0*M_PI*((float)n)/((float) DATA_BUF_LEN))));
			     break;
			  }
		    }
       }
}
#endif




cpu_init()
// performs DSP initialization
 {
	// clear & disable all IRQs
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	// perform common initialization:
	// 	assign INT4    = McBSP-0 Transmitter interrupt
	// 	assign INT5    = McBSP-0 Receiver interrupt
	if(   C6X_INTMUX_SET(4,C6X_INTSEL_MCBSP0_XINT)!=MERR_OK
	   || C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP0_RINT)!=MERR_OK
	   ) return(-1);



	// perform TORNADO-E6x DSP System initialization, set DSP type
#if(_E6X_SELECTED)

		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);
#endif


	// perform TORNADO-6x DSP System initialization, set DSP type
#if(_T6X_SELECTED)

		// init DSP EMIF
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif


	// perform TORNADO-P6x DSP System initialization, set DSP type
#if(_TP6X_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif

	// reset DCM in order to reset synchronization in case application has been
	// restarte several times without DSP hardware reset.
	_SET_DCM_RESET;
	_wait_for_dcm_reset();

	return(0);
 }




int	cmnd_decoder()
// decodes command
 {
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
       	// application specific commands
	   case MCMND_DAC_SINE_OUTPUT:
	   case MCMND_ADC_TO_DAC:
	   	 return( daq_run() );

		// all unknown commands cause error
	   default:			
		 return( MERR_INV_CMND );
	 }
 }






void main()
{ 
	// first, disable IRQs if this is the program restart under emultor 
	// without DSP reset
	C6X_GINT_DISABLE;

	// init CPU and allocate ADC/DAC buffers fm the heap
	if(   cpu_init()!=MERR_OK
	   || (ADC=(int *) malloc(DATA_BUF_LEN*2L*4L))==NULL
	   || (DAC=(int *) malloc(DATA_BUF_LEN*2L*4L))==NULL
	  )
		goto error;

	// set MCD params from the .CFG file
	MCD->epoch_len		= DATA_BUF_LEN;		// default AD/DA bufer length to '0'
	MCD->status			= MSTAT_RDY;

	MCD->cmnd			= _FUNC_;
	MCD->aic_ra			= _AIC_RA_;
	MCD->aic_rb			= _AIC_RB_;
	MCD->aic_ain_mux	= _AIC_AIN_MUX_;
	MCD->aic_ain_gain	= _AIC_AIN_GAIN_;
	MCD->aic_aout_att	= _AIC_AOUT_ATT_;


	// fill-in DAC data buffers with SINE data for DAC SINE OUTPUT demo function
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	MCD->err = cmnd_decoder();	// call exec & return error code
    MCD->status = MSTAT_RDY;	// clear status flag


	// get here on any error
error:
	while(1);
}
